/* ǡڤġ
 * 05/29 2000
 * shige
 * aucut.c
 *
 * $Id: aucut.c,v 1.7 2017-03-09 17:17:52+09 shige Exp $
 */
#ifndef __AUCUT_C__
#define __AUCUT_C__

#undef DEBUG1
#undef DEBUG2
#undef DEBUG3

#include "aucut.h"

int main(int argc, char *argv[])
{
    FILE *infile, *outfile = NULL;
    AUdata data;
    int j, max, sounds, n;
    Aupos h[MaxAufiles];
    double R, T, NT;
    char *path, *aufnames[MaxAufiles];

    if ((n = getarg(argc, argv, &R, &T, &NT, &path, aufnames, &infile))
	<= 0) {
	usage(); 
	return Exit_getarg;
    }
    if ((j = getAUheader(infile, &data, 0) < 0)) {
	if (verbose)
	  fprintf(stderr, "Error in getAUheader(). (%d)\n", j);
	return Exit_getSNDheader;
    }
    if ((max = getmaxdata(&data, infile)) <= 0) {
	if (verbose) fprintf(stderr, "This has no sound data.\n");
	return Exit_getmaxdata;
    } else if (max >= Maxlevel) {
	if (verbose)
	  fprintf(stderr, "This has Maxlevel sound data.\n");
#ifdef MaxLevelStop
	return Exit_Maxlevel;
#endif
    };
    if ((sounds = getposdata(&data, infile, max, R, T, NT, h)) < 0) {
	if (verbose)
	  fprintf(stderr, "Error in getposdata(). (%d\n", sounds);
	return Exit_getposdata;
    }
    if (sounds != n) {
	if (verbose) fprintf(stderr, "%d sounds found:\n", sounds);
	return Exit_n_neq_sounds;
    }
    for (j=0; j<sounds; j++) {
#ifdef DEBUG1
	fprintf(stderr, "Pass 0.\n");
#endif
	if (openpathfile(&outfile, aufnames[j], path) < 0)
	  return Exit_openpathfile;
	soundfcut(infile, outfile, &h[j], &data);
	fclose(outfile);
    }
    fclose(infile);
    return No_err;
}


/* ##########  ########## */
void soundfcut(FILE *infile, FILE *outfile, Aupos *pos, AUdata *InSnd)
{
    int len;

#ifdef DEBUG3
    fprintf(stderr, "(pos, len) = (%d, %d)\n", pos->pos, pos->len);
#endif
    len = InSnd->datalen;
    InSnd->datalen = pos->len;
    putAUheader(outfile, InSnd);
    putfile2file(infile, outfile, pos->pos, pos->len);
    InSnd->datalen = len;
}

/* for DEBUG */
int putlineardata(AUdata *Snd, FILE *file)
{
    int j, k = 0;
    short int x;
    
    fseek(file, Snd->hlen, SEEK_SET);
    for (j=1; j<=Snd->datalen; j++) {
	x = ulaw2linear(fgetc(file));
	printf("%d %d\n", k++, x);
    }
    return No_err;
}

int openpathfile(FILE **Outfile, char *fname, char *path)
{
    char buff[MaxPathBuff];
    
#ifdef DEBUG1
    fprintf(stderr, "Pass 0.5.\n");
    fprintf(stderr, "path = [%s]\n", path);
    fprintf(stderr, "fname = [%s]\n", fname);
#endif
    if (strlen(path) + 1+strlen(fname) + 1 > MaxPathBuff)
      return Err_longpath;
    strcpy(buff, path);
    strcat(buff, PathDelimiter);
    strcat(buff, fname);
    strcat(buff, AuFileExt);
    if (*Outfile != NULL) fclose(*Outfile);
    if ((*Outfile = fopen(buff, "w")) == NULL) return Err_fopen;
    return No_err;
}

/* ##########  ########## */
/* 1 path 
 * ͤκ֤ͤ
 */
int getmaxdata(AUdata *Snd, FILE *file)
{
    int j, max = 0, x;
    
    fseek(file, Snd->hlen, SEEK_SET);
    for (j=1; j<=Snd->datalen; j++) {
	x = ulaw2linear(fgetc(file));
	if (x < 0) x = -x;
	if (x > max) max = x;
    }
    return max;
}

/* 2 path 
 * S/N Τ R ̵ʬκû NT Ȥäƻΰ֤
 * n:  data θĿ
 * T: û data
 */
int getposdata(AUdata *Snd, FILE *file, int max, double R, double T,
	       double NT, Aupos h[])
{
    int j;
    int innoise, soundnum, noiselen, soundlen, pos;
    int Minnoiselen, Minsoundlen;
    double maxR;
    short int x;

    if (max <= 0 || R <= 0.0 || NT <= 0.0 || T < 0.0) return Err_param;
    Minnoiselen = (int)(NT*SAMPLES) + 1;
    Minsoundlen = (int)(T*SAMPLES) + 1;
    fseek(file, Snd->hlen, SEEK_SET);

    x = ulaw2linear(fgetc(file));
    maxR = max*R;
    if (x <= maxR && x >= -maxR) { /* Υ */
	innoise = 1;
	noiselen = 1;
	soundnum = 0;
    } else {
	soundnum = 1;
#ifdef DEBUG2
	fprintf(stderr, "%d sound start\n", soundnum);
#endif
	pos = ftell(file);
	soundlen = 1;
	innoise = 0;
    }
#ifdef DEBUG2
    fprintf(stderr, "(Minnoiselen, Minsoundlen) = (%d, %d)\n",
	    Minnoiselen, Minsoundlen);
    fprintf(stderr, "max = %d, x = %d, innoise = %d\n", max, x,
	    innoise);
    fprintf(stderr, "NR:(x, x/max, noise, innoise):\n");
#endif
    
    for (j=2; j<=Snd->datalen; j++) {
	x = ulaw2linear(fgetc(file));
#ifdef DEBUG2
	fprintf(stderr, "%d:(%d, %f, %d, %d)\n", j, x,
		(double)x/(double)max,
		(x <= maxR && x>= - maxR) ? 1 : 0, innoise);
#endif
	if (x <= maxR && x >= -maxR) { /* Υ */
	    switch (innoise) {
	    case 2: /* ʥΥ */
		break;
	    case 1: /* ΰΤʤ */
		if (++noiselen > Minnoiselen) {
		    if (soundnum > 0) {
			if (soundlen >= Minsoundlen) {
#ifdef DEBUG2
			    fprintf(stderr, "%d sound end\n",
				    soundnum);
#endif
			    h[soundnum - 1].pos = pos;
			    h[soundnum - 1].len = soundlen;
			} else {
#ifdef DEBUG2
			    fprintf(stderr, "Not a %d sound\n",
				    soundnum);
#endif
			    soundnum--;
			}
		    }
		    innoise = 2;
		}
		break;
	    case 0: /*  */
		innoise = 1;
		noiselen = 1;
		break;
	    }
	}
	else {
	    switch (innoise) {
	    case 2: 
		if (++soundnum > MaxAufiles)
		  return Err_too_many_sounds;
#ifdef DEBUG2
		fprintf(stderr, "%d sound start\n", soundnum);
#endif
		soundlen = 1;
		pos = ftell(file);
		innoise = 0;
		break;
	    case 1: 
		if (soundnum > 0)
		  soundlen += noiselen + 1;
		innoise = 0;
		break;
	    case 0:
		soundlen++;
		break;
	    }
	}
    }

    switch (innoise) {
    case 1: 
	if (soundnum > 0) {
	    soundlen += noiselen;
	    if (soundlen >= Minsoundlen) {
#ifdef DEBUG2
		fprintf(stderr, "%d sound end\n", soundnum);
#endif
		h[soundnum - 1].pos = pos;
		h[soundnum - 1].len = soundlen;
	    } else
	      soundnum--;
	}
	break;
    case 0:
	if (soundnum > 0) {
	    if (soundlen >= Minsoundlen) {
#ifdef DEBUG2
		fprintf(stderr, "%d sound end\n", soundnum);
#endif
		h[soundnum - 1].pos = pos;
		h[soundnum - 1].len = soundlen;
	    } else
	      soundnum--;
	}
	break;
    }
    return soundnum;
}

/* ########## option parameter  ########## */
int getarg(int argc, char *argv[], double *R, double *T, double *NT,
	   char **Path, char *aufnames[], FILE **Infile)
{
    int j, ret;
    char c;
    double x;

    *R = Default_R;
    *T = Default_T;
    *NT = Default_NT;
    *Path = Default_Path;
    
    if (argc < NeedOptions + 1) return Err_argnum;
    j = 1;
    while (*argv[j] == '-') {
	if ((c = *(++argv[j])) == 'r' || c == 't' || c == 's') {
	    if (*(++argv[j]) == '\0' && ++j >= argc - NeedOptions)
	      return Err_param;
	    if ((x = atof(argv[j])) <= 0.0) return Err_bad_value;
	    switch (c) {
	    case 'r': *R = x; break;
	    case 't': *T = x; break;
	    case 's': *NT = x; break;
	    }
	}
	else if (c == 'p') {
	    if (*(++argv[j]) == '\0' && ++j >= argc - NeedOptions)
	      return Err_param;
	    *Path = argv[j];
	} else if (c == 'v') {
	    if (*(++argv[j]) != '\0') return Err_param;
	    verbose = 1;
	} else return Err_unknown_option;
	++j;
    }
    if (j != argc - NeedOptions) return Err_argnum;
    if ((ret = op2aufnames(aufnames, MaxAufiles, argv[j++])) < 0)
      return ret;
    if ((*Infile = fopen(argv[j], "r")) == NULL) return Err_fopen;
    return ret;
}

/* 겻ե̾ fnames[]  ( max ) ¸  */
int op2aufnames(char *fnames[], int max, char *option)
{
    int flag = 0, len = 0, num = 0, j;
    char *ss, sh0[MaxNameLen], sf0[MaxNameLen];
    int start, end;

    ss = sh0;
    while (1) {
	switch (*option) {
	case '-':
	    *ss = '\0';
	    ss = sf0;
	    len = 0;
	    flag = 1;
	    break;
	case ',':
	case '\0':
	    *ss = '\0';
	    if (flag) { /* [a] - [b] */
		if ((start = searchautable(sh0)) < 0 
		    || (end = searchautable(sf0)) < 0 || start > end)
		  return Err_bad_value;
		if (num + end - start + 1 > max)
		  return Err_too_many_sounds;
		for (j=start; j<=end; j++)
		  fnames[++num - 1] = aufiletable[j];
		flag = 0;
	    } else { /* [a] */
		if ((start = searchautable(sh0)) < 0)
		  return Err_bad_value;
		if (num + 1 > max) return Err_too_many_sounds;
		fnames[++num - 1] = aufiletable[start];
	    }
	    if (*option == '\0') return num;
	    len = 0;
	    ss = sh0;
	    break;
	default:
	    if (++len >= MaxNameLen) return Err_bad_value;
	    *(ss++) = *option;
	    break;
	}
	option++;
    }
}

/* aufiletable[] 鳺򸡺 (0~) */
int searchautable(char *s)
{
    int j;

    if (strlen(s) <= 0) return Err_bad_value;
    for (j=0; strcmp(aufiletable[j], AUFTABLELAST)!=0; j++)
      if (strcmp(s, aufiletable[j]) == 0) return j;
    return Err_bad_value;
}

void usage(void)
{
    char *mess[] = {
	"Usage: aucut [options] [ϥե] [aufile]",
	" options:",
	"   -r[R] : S/N Τ (default: 0.1)",
	"   -t[T] : ûǡĹ [] (default: 0.01)",
	"   -s[S] : ûʬĹ [] (default: 0.01)",
"   -p[path]: ϥեǼѥ (default: ȥǥ쥯ȥ)",
	"   -v    : Ĺ",
	" ",
	" [1] ϥե",
	"     (a)  ',' ڤ (: a,o,ko)",
	"     (b)  '-' ڤ (: a-ko ==> a,i,u,e,o,ka,ki,ku,ke,ko)",
	"     νǤ롣ڡʤȡ",
	" [2] (ζκ)(S/N Τ) ʲζΥǡ",
	"     ʬϻʬȤߤʤ롣ä",
	"      ==> 򽦤䤹ʤꡢƲĹˤʤꡢ",
	"                    ",
	"     礭 ==> ȸʤƼΤƤƤޤ",
	"                    Ʋûˤʤꡢϸ",
	" [3] ûǡĹûʬϻȸʤä",
	"      ==> ԡΥΥ򽦤䤹ʤ",
	"     礭 ==> ûΥȸƼΤƤƤޤ",
	" [4] ǡλʬǡûʬĹûʬϰŪ",
	"     ǡ夯ʤäƤʬȽǤǡ³Ƥ",
	"     ʤ",
	"      ==> Ū˼夤ʬⲻڤܤȽǤƲ",
	"                    ڤäƤޤ",
	"     礭 ==> ȲȤδ֤ⲻĤʤäƤȸʤƲ",
	"                    ڤʤʤ",
	NULL
    };
    int j;

    for (j=0; mess[j]!=NULL; j++)
      fprintf(stderr, "%s\n", mess[j]);
}

#endif /* __AUCUT_C__ */
