/* ǡ򥯥å/ǥå
 * 04/22 2008
 * shige
 * sndcresc.c
 *
 * $Id: sndcresc.c,v 1.4 2017-03-23 19:27:33+09 shige Exp $
 */
/* sndcresc (-o [outfile]) -{s,e}v[n] ([infile])
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "common.h"

#define Voldelimit ':'
#define Maxchannel 10

int main(int argc, char *argv[]);
int setvolume(char *str, double vol[]);
double linearvol(double sv, double ev, long int j, long int blocks);
int getarg(int argc, char **argv, double svol[], double evol[],
	   FILE **infile, FILE **outfile);
void usage(int ret);

int main(int argc, char *argv[])
{
    SNDheader snd;
    FILE *infile, *outfile;
    int ret, format, ch, bs, k;
    double x, svol[Maxchannel], evol[Maxchannel], v;
    long int j, blocks;
    
    if ((ret = getarg(argc, argv, svol, evol, &infile, &outfile)) 
	<= 0) {
	usage(ret); return -ret;
    }
    if ((format = getSNDheader(infile, &snd)) < 0 
	|| format == E_UNKNOWN) {
	fprintf(stderr, "Error: Read getSNDheader(). (%d)\n", format);
	return 1;
    }

    ch = SNDchannels(&snd, format);

#ifdef DEBUG1
    fprintf(stderr, "return = %d\n", ret);
    for (k=0; k<ch; k++)
      fprintf(stderr, "(sv[%d], ev[%d]) = (%f, %f)\n", k, k
	      svol[k], evol[k]);
    fprintf(stderr, " == ===\n");
    for (; j<Maxchannel; j++)
      fprintf(stderr, "(sv[%d], ev[%d]) = (%f, %f)\n", k, k
	      svol[k], evol[k]);
#endif

    if (ret - 1 > ch || ch > Maxchannel) { 
	fprintf(stderr, "Channel number error. (%d, %d, %d)\n",
		ret - 1, ch, Maxchannel);
	return 2;
    }

    putSNDheader(outfile, &snd, format);
    blocks = SNDblocks(&snd, format);
    bs = SNDblocksize(&snd, format);
    for (j=1; j<=blocks; j++)
      for (k=0; k<ch; k++) {
	  v = linearvol(svol[k], evol[k], j, blocks);
	  if (v == 0.0) SNDputnull1ch(outfile, &snd, format);
	  else if (v == 1.0)
	    SNDpass1chrawdata(infile, outfile, &snd, format);
	  else {
	      x = SNDget1chleveldata(infile, &snd, format);
	      if ((x *= v) > 1.0) x = 1.0;
	      else if (x < -1.0) x = -1.0;
	      SNDput1chleveldata(outfile, x, &snd, format);
	  }
      }
    if (format != E_AU && bs%2 && blocks%2) fputc(0, outfile);
    return 0;
}

double linearvol(double sv, double ev, long int j, long int blocks)
{
    return sv + (ev - sv)*(j - 1L)/(blocks - 1L);
}

int getarg(int argc, char **argv, double svol[], double evol[],
	   FILE **infile, FILE **outfile)
{
    int j, maxchannel = 0;

    *outfile = stdout;
    *infile = stdin;

    for (j=0; j<Maxchannel; j++) { svol[j] = evol[j] = 1.0; }
    j = -1;
    while (-- argc > 0) {
	if (**(++ argv) == '-') {
	    switch (*(++ *argv)) {
	    case 's': 
		if (*(++ *argv) != 'v') return -10; 
		if (*(++ *argv) == '\0') { 
		    if (-- argc <= 0) return -1;
		    else argv ++;
		}
		if ((j = setvolume(*argv, svol)) < 0) return -2;
		if (j > maxchannel) maxchannel = j;
		break;
	    case 'e': 
		if (*(++ *argv) != 'v') return -10; 
		if (*(++ *argv) == '\0') { 
		    if (-- argc <= 0) return -1;
		    else argv ++;
		}
		if ((j = setvolume(*argv, evol)) < 0) return -2;
		if (j > maxchannel) maxchannel = j;
		break;
	    case 'h':
	    case 'H':
	    case '?': return 0;
	    case 'o':
		if (*outfile != stdout) return -3;
		if (*(++ *argv) == '\0') { 
		    if (-- argc <= 0) return -4;
		    else argv++;
		}
		if (!strcmp(*argv, " - ")) *outfile = stdout;
		else if ((*outfile = fopen(*argv, "w")) == NULL) {
		    fprintf(stderr, "File [%s] open error.\n", *argv);
		    return -5;
		}
		break;
	    case '\0': break;
	    default: return -6;
	    }
	    if (**argv == '\0') { -- *argv; break; }
	} else break;
    }
    if (argc > 1 || j < 0) return -7;
    else if (argc < 1) return maxchannel + 1;
    else if (!strcmp(*argv, "-")
	     || (*infile = fopen(*argv, "r")) != NULL) {
	return maxchannel + 1;
    } else {
	fprintf(stderr, "File [%s] open error.\n", *argv);
	return -9;
    }
}

int setvolume(char *str, double vol[])
{
    int len = strlen(str), j, k, ch;
    char *s, *s1;
    double x;

    if (len <= 0) return -1;
    s = str; k = 0; ch = 0;
    for (j=1; j<=len; j++) {
	if (*s == Voldelimit) {
	    if (++k >= 2) return -2;
	    s1 = s;
	}
	s++;
    }
    /* k == 0 ʤ ch = 0 Ʊ */
    if (k == 0) { ch = 0; x = atof(str); }
    else {
	/* k == 1 ʤ s ΰ֤ '\0' ˤͤ */
	if (*(s1 + 1) == '\0') return -3;
	if (*str == '\0') ch = 0;
	else if ((ch = atoi(str)) > Maxchannel || -ch > Maxchannel)
	  return -4;
	x = atof(s1 + 1);
    }
    /* x  vol Ǥϼ¿ */
    if (ch == 0) { 
	for (j=1; j<=Maxchannel; j++) vol[j - 1] = x;
	return 0;
    } else if (ch > 0) { 
	vol[ch - 1] = x; return ch; 
    } else {
	for (j=1; j<=Maxchannel; j++) if (j != -ch) vol[j - 1] = x;
	return -ch;
    }
}

void usage(int ret)
{
    char *mess[] = {
	"Usage: sndcresc (-o [outfile]) (-{s,e}v[n1] -{s,e}[n2] ...) ([infile])",
	"   -o [outfile]: ϥեλ (default: ɸ)",
	"   -sv[string]:  volume λ (default: 1.0)",
	"   -ev[string]: λ volume λ (default: 1.0)",
	"   [infile]: ϥե (ά: ɸ)",
	" ",
	" [1] ϥե '-' ꤷɸϤˤʤ롣",
	" [2] ǡβ̤ܤ롣ͤϡ",
	"     Ƭǳ volume ͡ǸϽλ volume ͤȤʤ褦",
	"     Ѳ롣ꤷʤä channel ϲ⤷ʤ",
	" [3] volume ϰʲ̤:",
	"       -sv[vol]     : ٤Ƥ channel  [vol] ܤ鳫",
	"       -sv[n]:[vol] : [n] channel Τߤ [vol] ܤ鳫",
	"     [n] ϥֹͥ桢[vol] Ǥդμ¿ (ͤ)",
	"     [n] < 0 ΤȤϡ|[n]| ͥʳλˤʤ롣",
	"     [vol] < 0.0 ΤȤϰ꤬ȿž롣",
	" [4] -sv,-ev ϸλ꤬λ񤭤롣ä㤨",
	"        -sv2 -sv1:-0.5 -sv1:1.2",
	"     ϡ1 channel  1.2 ܡ¾ channel  2 ܤ鳫Ϥ롣",
	NULL
    };
    int j;

    for (j=0; mess[j]!=NULL; j++) fprintf(stderr, "%s\n", mess[j]);
}
